@@ -13,13 +13,17 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import pkg_resources
+import stevedore
 from testtools import matchers
 
 from glance_store import backend
 from glance_store.tests import base
 
 
+def on_load_failure_callback(*args, **kwargs):
+    raise
+
+
 class OptsTestCase(base.StoreBaseTest):
 
     def _check_opt_groups(self, opt_list, expected_opt_groups):
@@ -41,11 +45,16 @@ class OptsTestCase(base.StoreBaseTest):
     def _test_entry_point(self, namespace,
                           expected_opt_groups, expected_opt_names):
         opt_list = None
-        for ep in pkg_resources.iter_entry_points('oslo.config.opts'):
-            if ep.name == namespace:
-                list_fn = ep.load()
-                opt_list = list_fn()
-                break
+        mgr = stevedore.NamedExtensionManager(
+            'oslo.config.opts',
+            names=[namespace],
+            invoke_on_load=False,
+            on_load_failure_callback=on_load_failure_callback,
+        )
+        for ext in mgr:
+            list_fn = ext.plugin
+            opt_list = list_fn()
+            break
 
         self.assertIsNotNone(opt_list)
 
@@ -58,20 +67,24 @@ class OptsTestCase(base.StoreBaseTest):
         expected_opt_names = [
             'default_store',
             'stores',
-            'store_capabilities_update_min_interval',
             'cinder_api_insecure',
             'cinder_ca_certificates_file',
             'cinder_catalog_info',
             'cinder_endpoint_template',
             'cinder_http_retries',
+            'cinder_mount_point_base',
             'cinder_os_region_name',
             'cinder_state_transition_timeout',
             'cinder_store_auth_address',
             'cinder_store_user_name',
             'cinder_store_password',
             'cinder_store_project_name',
+            'cinder_volume_type',
+            'cinder_use_multipath',
+            'cinder_enforce_multipath',
             'default_swift_reference',
             'https_insecure',
+            'filesystem_store_chunk_size',
             'filesystem_store_datadir',
             'filesystem_store_datadirs',
             'filesystem_store_file_perm',
@@ -89,20 +102,11 @@ class OptsTestCase(base.StoreBaseTest):
             's3_store_bucket_url_format',
             's3_store_create_bucket_on_put',
             's3_store_host',
-            's3_store_object_buffer_dir',
             's3_store_secret_key',
             's3_store_large_object_size',
             's3_store_large_object_chunk_size',
             's3_store_thread_pools',
-            's3_store_enable_proxy',
             'swift_store_expire_soon_interval',
-            's3_store_proxy_host',
-            's3_store_proxy_port',
-            's3_store_proxy_user',
-            's3_store_proxy_password',
-            'sheepdog_store_address',
-            'sheepdog_store_chunk_size',
-            'sheepdog_store_port',
             'swift_store_admin_tenants',
             'swift_store_auth_address',
             'swift_store_cacert',
@@ -124,7 +128,10 @@ class OptsTestCase(base.StoreBaseTest):
             'swift_store_ssl_compression',
             'swift_store_use_trusts',
             'swift_store_user',
-            'vmware_api_insecure',
+            'swift_buffer_on_upload',
+            'swift_upload_buffer_dir',
+            'vmware_insecure',
+            'vmware_ca_file',
             'vmware_api_retry_count',
             'vmware_datastores',
             'vmware_server_host',
